local faction = UnitFactionGroup("player");
local playerName = UnitName("player");
local realmName = GetRealmName();

local HasVersionControlled = false;
WorldBuffTrackerConfigurableVars = {};
local WorldBuffTrackerVars = {
  ["PlaySound"] = true,
  ["SelfWhisper"] = true,
  ["GuildAnnounce"] = false,
}
local _, L = ...;

function WBT_GetBuffData(buff, data)
  return WorldBuffTracker[realmName][faction][buff][data];
end

function WorldBuffTracker_VerifyTable()
  for k, v in pairs(WorldBuffTrackerVars) do
    if(WorldBuffTracker[k] == nil and type(v) == "boolean") then
      WorldBuffTracker[k] = v;
      WorldBuffTrackerConfigurableVars[k] = v;
    end
  end
  if(type(WorldBuffTracker[realmName]) ~= "table") then
    WorldBuffTracker[realmName] = {};
    WorldBuffTracker[realmName]["Characters"] = {};
    if(type(WorldBuffTracker[realmName][faction]) ~= "table") then
      WorldBuffTracker[realmName][faction] = {};
      WorldBuffTracker[realmName][faction]["LastSync"] = 0;
    end
  end
  WorldBuffTracker_TestBuffSubtable("1");
  WorldBuffTracker_TestBuffSubtable("2");
  WorldBuffTracker_TestBuffSubtable("3");
end

function WorldBuffTracker_TestBuffSubtable(buff)
  local buffEntryType = type(WorldBuffTracker[realmName][faction][buff]);
  if(buffEntryType ~= "table") then
    WorldBuffTracker[realmName][faction][buff] = {
      ["Timestamp"] = 0,
      ["Yell"] = 0,
      ["Version"] = WorldBuffTracker_GetVersion(),
      ["Sender"] = UnitName("player"),
      ["Origin"] = UnitName("player"),
      ["Killed"] = 0,
      ["Y0"] = 0,
      ["Y1"] = 0,
      ["Y2"] = 0,
      ["Weight"] = 0,
    }
  end
  if(type(WorldBuffTracker[realmName][faction][buff]["Killed"]) == "nil") then
    WorldBuffTracker[realmName][faction][buff]["Killed"] = 0;
  end
  if(type(WorldBuffTracker[realmName][faction][buff]["Timestamp"]) == "nil") then
    WorldBuffTracker[realmName][faction][buff]["Timestamp"] = 0;
  end
end

function WorldBuffTracker_DistinctTimestamp(list, timestamp, weight, currentTime)
  if(type(WorldBuffTracker[realmName][faction][list]) == "table" and tonumber(list) ~= nil) then
    if(abs(timestamp-WorldBuffTracker[realmName][faction][list]["Timestamp"]) < 30) then
      return false;
    end
    if(timestamp-WorldBuffTracker[realmName][faction][list]["Timestamp"] + WorldBuffTrackerBuffCDs[list] > currentTime
      and WorldBuffTracker[realmName][faction][list]["Weight"] > weight) then
      return false;
    end
    return true;
  end
  return false;
end

function WorldBuffTracker_HandleSync(message, sender, guild, channel)
  local version, b1, b2, b3 = strsplit("_", message);
  if(b1 == nil) then return end
  version = WorldBuffTracker_GetVersionFromString(version);
  local t = GetServerTime();
  local x1 = WorldBuffTracker_HandleSyncSubType("1", b1, sender, guild, t, channel);
  local x2 = WorldBuffTracker_HandleSyncSubType("2", b2, sender, guild, t, channel);
  local x3 = WorldBuffTracker_HandleSyncSubType("3", b3, sender, guild, t, channel);
  if(x1 or x2 or x3) then
    WorldBuffTracker_BroadcastFreshData(channel);
  end
end

function WorldBuffTracker_HandleSyncSubType(buffType, string, sender, guild, currentTime, channel)
  if(string == nil) then return end
  print(string);
  local timestamp, origin, killed, weight, version = strsplit(":", string);
  weight = tonumber(weight);
  if(not guild and weight > 2) then
    weight = 2;
    return;
  end
  timestamp = WorldBuffTracker_HexToDec(timestamp);
  killed = WorldBuffTracker_HexToDec(killed);
  version = WorldBuffTracker_GetVersionFromString(WorldBuffTracker_HexToDec(version));
  if(timestamp == nil or timestamp > currentTime
    or (WorldBuffTracker[realmName][faction][buffType]["Timestamp"] > 0 and
    version < WorldBuffTracker[realmName][faction][buffType]["Version"])) then
    return
  end
  if(WorldBuffTracker_DistinctTimestamp(buffType, timestamp, weight, currentTime)) then
    if(WorldBuffTracker[realmName][faction][buffType]["Version"] <= version or WorldBuffTracker[realmName][faction][buffType]["Timestamp"] == 0) then
      WorldBuffTracker[realmName][faction][buffType]["Timestamp"] = time;
      WorldBuffTracker[realmName][faction][buffType]["Sender"] = sender;
      WorldBuffTracker[realmName][faction][buffType]["Origin"] = origin;
      WorldBuffTracker[realmName][faction][buffType]["Killed"] = killed;
      WorldBuffTracker[realmName][faction][buffType]["Weight"] = weight;
      WorldBuffTracker[realmName][faction][buffType]["Version"] = version;
      WBT_GUI_TestBar(tonumber(buffType), currentTime);
      return true;
    end
  end
end

function WorldBuffTracker_SendSyncData(target)
  C_ChatInfo.SendAddonMessage("WBT-S", WorldBuffTracker_BuildSyncMessage(), "WHISPER", target);
end

function WorldBuffTracker_BuffNPCDeath(npc)
  local t,p = GetServerTime(), UnitName("player");
  WorldBuffTracker[realmName][faction][buffType]["Killed"] = t;
  WorldBuffTracker[realmName][faction][buffType]["Timestamp"] = t;
  WorldBuffTracker[realmName][faction][buffType]["Origin"] = p;
  WorldBuffTracker[realmName][faction][buffType]["Sender"] = p;
  WorldBuffTracker[realmName][faction][buffType]["Weight"] = 3;
  WorldBuffTracker[realmName][faction][buffType]["Version"] = WorldBuffTracker_GetVersion();
  WorldBuffTracker_BroadcastFreshData();
  --ANNOUNCE
end

function WorldBuffTracker_BroadcastFreshData(channelRecieved)
  local data = WorldBuffTracker_BuildSyncMessage();
  if(IsInGuild() and channelRecieved ~= "GUILD") then
    WorldBuffTracker_BroadcastChannel(data, "GUILD");
  end
  if(IsInGroup() and (channelRecieved ~= "PARTY" and channelRecieved ~= "RAID")) then
    WorldBuffTracker_BroadcastChannel(data, (IsInRaid() and "RAID" or "PARTY"));
  end
  if(not IsInInstance() and channelRecieved ~= "YELL") then
    WorldBuffTracker_BroadcastChannel(data, "YELL");
  end
end

function WorldBuffTracker_BroadcastChannel(data, channel)
  C_ChatInfo.SendAddonMessage("WBT-S", data, channel);
end

function WorldBuffTracker_BroadcastIncomingData(channelRecieved, buffID, timestamp, weight, yell)
  local data = WorldBuffTracker_Serialize(WorldBuffTracker_GetVersion(), buffID, timestamp, weight, yell);
  if(IsInGuild() and channelRecieved ~= "GUILD") then
    WorldBuffTracker_BroadcastIncomingChannel(data, "GUILD");
  end
  if(IsInGroup() and (channelRecieved ~= "PARTY" and channelRecieved ~= "RAID")) then
    WorldBuffTracker_BroadcastIncomingChannel(data, (IsInRaid() and "RAID" or "PARTY"));
  end
  if(not IsInInstance() and channelRecieved ~= "YELL") then
    WorldBuffTracker_BroadcastIncomingChannel(data, "YELL");
  end
end

function WorldBuffTracker_BroadcastIncomingChannel(data, channel)
  C_ChatInfo.SendAddonMessage("WBT-I", data, channel);
end

function WorldBuffTracker_RequestGroup()
  C_ChatInfo.SendAddonMessage("WBT-R", "yea", (IsInRaid() and "RAID" or "PARTY"));
end

function WorldBuffTracker_BuildSyncMessage()
  return WorldBuffTracker_Serialize(WorldBuffTracker_GetVersion(),
    WorldBuffTracker_BuildSyncMessageSubType("1"),
    WorldBuffTracker_BuildSyncMessageSubType("2"),
    WorldBuffTracker_BuildSyncMessageSubType("3"));
end

function WorldBuffTracker_BuildSyncMessageSubType(id)
  return WorldBuffTracker_DecToHex(WorldBuffTracker[realmName][faction][id]["Timestamp"]) .. ":" ..
    WorldBuffTracker_GetTrueOrigin(id) .. ":" ..
    WorldBuffTracker_DecToHex(WorldBuffTracker[realmName][faction][id]["Killed"]) .. ":" ..
    WorldBuffTracker[realmName][faction][id]["Weight"] .. ":" ..
    WorldBuffTracker_DecToHex(WorldBuffTracker[realmName][faction][id]["Version"]);
end

function WorldBuffTracker_GetTrueOrigin(id)
  local origin = WorldBuffTracker[realmName][faction][id]["Origin"];
  if(WorldBuffTracker[realmName]["Characters"][origin] == nil) then
    return origin;
  else
    return playerName;
  end
end

function WorldBuffTracker_GetOriginString(v)
  local sources = " ";
  if(WorldBuffTracker[realmName][faction][v]["Origin"] ~= WorldBuffTracker[realmName][faction][v]["Sender"]) then
    sources = sources .. "(" .. WorldBuffTracker[realmName][faction][v]["Origin"] .. "→" .. WorldBuffTracker[realmName][faction][v]["Sender"] .. ")";
  else
    sources = sources .. "(" .. WorldBuffTracker[realmName][faction][v]["Sender"] .. ")";
  end
  return sources;
end

function WorldBuffTracker_HandleBroadcast(message, sender)
  local id, time, version = strsplit("_", message);
  if(id == nil) then return end
  if(id ~= "1" and id ~= "2" and id ~= "3") then return end
  WorldBuffTracker_HandleBuffByType(id, sender, WorldBuffTracker_GetVersionFromString(version), tonumber(time));
end

function WorldBuffTracker_HandleBuffById(id)
  WorldBuffTracker_HandleBuffByType(id, UnitName("player"), WorldBuffTracker_GetVersion(), GetServerTime());
end

function WorldBuffTracker_HandleBuffByType(type, sender, version, time)
  if(version < 11330) then
    return
  end
  if(WorldBuffTracker[realmName][faction][type]["Timestamp"] + WorldBuffTrackerBuffCDs[type] < time) then
    WorldBuffTracker[realmName][faction][type]["Timestamp"] = time;
    WorldBuffTracker[realmName][faction][type]["Version"] = version;
    WorldBuffTracker[realmName][faction][type]["Sender"] = sender;
    WorldBuffTracker[realmName][faction][type]["Origin"] = sender;
    WBT_GUI_TestBar(tonumber(type), time);
  elseif(WorldBuffTracker[realmName][faction][type]["Version"] < version) then
    WorldBuffTracker[realmName][faction][type]["Timestamp"] = time;
    WorldBuffTracker[realmName][faction][type]["Version"] = version;
    WorldBuffTracker[realmName][faction][type]["Sender"] = sender;
    WorldBuffTracker[realmName][faction][type]["Origin"] = sender;
    WBT_GUI_TestBar(tonumber(type), time);
  end
end

function WorldBuffTracker_GetVersionFromString(str)
  local v = tonumber(str) ~= nil and tonumber(str) or 11321;
  if(not HasVersionControlled and v > WorldBuffTracker_GetVersion()) then
    HasVersionControlled = true;
    print("|cFFB90AFF[WorldBuffTracker]|r |cFF30D4FF" .. L["A new update is available."] .. "|r");
  end
  return v;
end

function WorldBuffTracker_BuffIsValid(spell, type)
  for i=1,32 do
    local b,_,_,_,e,f,_ = UnitBuff("player", i);
    if(b == nil) then
      return false;
    end
    if(spell == b and type == 1 and f-GetTime() > 3598) then
      return true;
    elseif(spell == b and f-GetTime() > 7198) then
      return true;
    end
  end
  return false;
end

function WorldBuffTracker_DecToHex(number)
  return string.format("%x", number);
end

function WorldBuffTracker_HexToDec(hex)
  return tonumber(hex, 16);
end

function WorldBuffTracker_Serialize(...)
  local str = "";
  for k, v in ipairs({...}) do
    if(str ~= "") then
      str = str .. "_" .. v;
    else
      str = v;
    end
  end
  return str;
end
