function WBT_CreateOptions()
  local options = CreateFrame("Frame", "WBT_Options")
  options.name = "WorldBuffTracker";

  WBT_Options_CreateTitleLabel(0, 0, "Incoming buffs");
  WBT_Options_CreateCheckbox(0, 20, "Silence in Instances", "Mutes & disables whispers in instances.", "SilentInstances", true);
  WBT_Options_CreateCheckbox(0, 40, "Self Whisper", "Whispers yourself when a buff is incoming (Makes the window glow when tabbed out)", "SelfWhisper", true);
  WBT_Options_CreateCheckbox(0, 60, "Sound Alert", "Plays a sound when a buff is incoming.", "PlaySound", true);

  WBT_Options_CreateTitleLabel(160, 0, "Buff Bars");
  WBT_Options_CreateCheckbox(160, 20, "Enabled", "Enables the buff bars on screen.", "BarsEnabled", true);
  WBT_Options_CreateCheckbox(160, 40, "Hide in Instances", "Hides buff cooldowns in instances.", "HiddenInstances", true);
  WBT_Options_CreateCheckbox(160, 60, "Unlock", "Unlocks the buff bars, allowing you to move them.", "Unlocked", false);
  WBT_Options_CreateLabel(160, 80, "Scale:");
  WBT_Options_CreateInputbox(195, 80, "100", "Scale");

  WBT_Options_CreateTitleLabel(320, 0, "Bar Format (Requires reload)");
  WBT_Options_CreateCheckbox(320, 20, "Popped", "Shows the time a buff was popped.", "StartLabel", false);
  WBT_Options_CreateCheckbox(320, 40, "CD Over", "Shows the time the cooldown is over.", "EndLabel", true);
  WBT_Options_CreateCheckbox(320, 60, "Remaining", "Shows the remaining time of the cooldown.", "RemainLabel", true);
  WBT_Options_CreateLabel(320, 80, "Width:");
  WBT_Options_CreateInputbox(355, 80, "240", "Width");

  WBT_Options_CreateTitleLabel(0, 100, "Incoming Buff Bar");
  WBT_Options_CreateCheckbox(0, 120, "Enabled", "Enables the incoming buff bar.", "IncomingEnabled", true);
  WBT_Options_CreateCheckbox(0, 140, "Zone Specific", "Only enable the bar when in the buff's zone.", "IncomingZone", true);
  WBT_Options_CreateCheckbox(0, 160, "Hide in Instances", "Hides the bar in instances.", "IncomingInstances", true);
  WBT_Options_CreateLabel(2, 180, "Width:");
  WBT_Options_CreateInputbox(40, 180, "600", "IncomingWidth");
  WBT_Options_CreateLabel(2, 200, "Scale:");
  WBT_Options_CreateInputbox(40, 200, "100", "IncomingScale");

  InterfaceOptions_AddCategory(options);
end

function WBT_Options_CreateTitleLabel(xOffset, yOffset, text)
  local uiObject = WBT_Options:CreateFontString(nil, "Overlay");
  uiObject:SetPoint("TOPLEFT", xOffset + 16, -yOffset - 16);
	uiObject:SetTextColor(1, 0.8, 0);
  uiObject:SetFont("Fonts\\FRIZQT__.TTF", 16);
  uiObject:SetText(text);
end

function WBT_Options_CreateLabel(xOffset, yOffset, text)
  local uiObject = WBT_Options:CreateFontString(nil, "Overlay");
  uiObject:SetPoint("TOPLEFT", xOffset + 12, -yOffset - 18);
	uiObject:SetTextColor(1, 0.8, 0);
  uiObject:SetFont("Fonts\\FRIZQT__.TTF", 12);
  uiObject:SetText(text);
end

local checkboxCount = 0;
function WBT_Options_CreateCheckbox(xOffset, yOffset, title, description, variable, default)
  checkboxCount = checkboxCount + 1;
	local checkbutton = CreateFrame("CheckButton", "WBT_Checkbox" .. checkboxCount, WBT_Options, "ChatConfigCheckButtonTemplate");
  checkbutton.tooltip = description;
  checkbutton.tooltipStyle = 0;
  getglobal(checkbutton:GetName() .. 'Text'):SetText(title);
  if(BuffGUIVars[variable]) then
    checkbutton:SetChecked(BuffGUIVars[variable]);
  else
    BuffGUIVars[variable] = default;
    checkbutton:SetChecked(BuffGUIVars[variable]);
  end
	checkbutton:SetPoint("TOPLEFT", xOffset + 10, -yOffset - 16);
	checkbutton:SetScript("OnClick", function()
    if(variable ~= "Unlocked") then
      BuffGUIVars[variable] = checkbutton:GetChecked();
    else
      WBT_GUI_ToggleLock(checkbutton:GetChecked());
    end
	end);
	return checkbutton;
end

local inputboxCount = 0;
function WBT_Options_CreateInputbox(xOffset, yOffset, text, variable)
  inputboxCount = inputboxCount + 1;
	local inputbox = CreateFrame("EditBox", "WBT_Inputbox" .. inputboxCount, WBT_Options);
	inputbox:SetSize(40, 16);
  inputbox:SetFontObject("GameFontHighlight");
  inputbox:SetBackdrop({
  	bgFile = [[Interface\Buttons\WHITE8X8]],
  	edgeFile = [[Interface\Buttons\WHITE8X8]],
  	edgeSize = 1,
  })
  inputbox:SetBackdropColor(0, 0, 0, 0.5)
  inputbox:SetBackdropBorderColor(0.3, 0.3, 0.30, 0.80)
	inputbox:SetPoint("TOPLEFT", xOffset + 16, -yOffset - 16);
	inputbox:SetAutoFocus(false);
	inputbox:SetMultiLine(false);
	inputbox:SetMaxLetters(3);
  inputbox:SetNumeric();
  inputbox:SetJustifyH("CENTER");
	inputbox:SetJustifyV("CENTER");
	inputbox:ClearFocus();
	inputbox:SetScript("OnEnterPressed", function()
		inputbox:ClearFocus();
	end);
	inputbox:SetScript("OnEscapePressed", function()
		inputbox:ClearFocus();
	end);
	inputbox:SetScript("OnTextChanged", function()
    local n = tonumber(inputbox:GetText());
    if(n ~= nil) then
      if(n > 0) then
        BuffGUIVars[variable] = tonumber(inputbox:GetText());
        if(variable == "Scale" and tonumber(inputbox:GetText()) / 100 ~= WBT_BuffGUI:GetScale()) then
          BuffGUIVars.X = math.floor(((GetScreenWidth() + WBT_BuffGUI:GetWidth() ) / 2/ WBT_BuffGUI:GetScale()));
          BuffGUIVars.Y = math.floor(((GetScreenHeight() + 80 ) / 2/ WBT_BuffGUI:GetScale()));
          WBT_BuffGUI:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", BuffGUIVars.X, BuffGUIVars.Y);
        end
        WBT_GUI_UpdateSize();
      else
        inputbox:SetText("0");
      end
    end
	end);
  if(BuffGUIVars[variable]) then
   inputbox:SetText(BuffGUIVars[variable]);
  else
   inputbox:SetText(text);
  end
 inputbox:SetCursorPosition(0)
end
