local _, L = ...;
local VERSION = 11330;
local realmName = GetRealmName();
local playerName = UnitName("player");
function WorldBuffTracker_GetVersion()
  return VERSION;
end
local faction = "Alliance";
local wbtString = "[WorldBuffTracker] ";
local color = {
  ["BLUE"] = "|cFF30D4FF",
  ["ORANGE"] = "|cFFFFAA0A",
  ["PURPLE"] = "|cFFB90AFF",
  ["GREEN"] = "|cFF00B71D",
}
--local ZoneEvents = {
  --[0] = "COMBAT_LOG_EVENT_UNFILTERED",
  --[1] = "CHAT_MSG_MONSTER_YELL",
--}
WorldBuffTrackerFrame = CreateFrame("Frame");
local addonInitialized = 99999999999999;
local SyncDataRecieved = false;
local SyncDataRecievedTimestamp = 0;
WorldBuffTrackerEvents = {}
WorldBuffTracker = {}

WorldBuffTrackerZones = {
  ["Horde"] = {
    [1411] = true, [1454] = true, [1413] = true;
  },
  ["Alliance"] = {
      [1453] = true, [1429] = true;
    },
  ["NEUTRAL"] = {
    [1412] = true, [1429] = true, [1434] = true;
  },
}

WorldBuffTrackerBuffCDs = {
  ["1"] = 10800, ["2"] = 21600, ["3"] = 28800,
  ["4"] = 14400, ["SF"] = 1500;
}

WorldBuffTrackerQuests = {
  ["Alliance"] = {
    [7496] = 2,
    [7782] = 3,
  },
  ["Horde"] = {
    [4974] = 1,
    [7491] = 2,
    [7784] = 3,
  }
}

function WorldBuffTracker_Initialize()
  if(GetSpellBookItemName(1, "spell") ~= nil) then
    if(WorldBuffTrackerSpells[GetLocale()] ~= nil) then
      WorldBuffTrackerSpells = WorldBuffTrackerSpells[GetLocale()];
      WorldBuffTrackerNPCs = WorldBuffTrackerNPCs[GetLocale()];
    else
      WorldBuffTrackerSpells = WorldBuffTrackerSpells["enUS"];
      WorldBuffTrackerNPCs = WorldBuffTrackerNPCs["enUS"];
    end
    for k,v in pairs(WorldBuffTrackerSpells) do
      if(v == 2) then
        WorldBuffTrackerSpellsInverse["2"] = k .. ": (" .. L["Onyxia"] .. ")";
        WorldBuffTrackerSpellsInverse["3"] = k .. ": (" .. L["Nefarian"] .. ")";
      else
        WorldBuffTrackerSpellsInverse[tostring(v)] = k;
      end
    end
    WorldBuffTrackerSpellsInverse["5"] = L["ZandalarBuff"];
    faction, _ = UnitFactionGroup("player");
    WorldBuffTracker_VerifyTable();
    if(GetServerTime() - WorldBuffTracker[realmName][faction]["LastSync"]  > 300) then
      if(IsInGuild()) then
        C_ChatInfo.SendAddonMessage("WBT-S", VERSION, "GUILD");
      end
      WorldBuffTracker[realmName][faction]["LastSync"] = GetServerTime();
    end
    WorldBuffTracker[realmName]["Characters"][playerName] = 0;
    addonInitialized = GetTime() + 0.5;
    WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_DelayTracking);
  end
end

function WorldBuffTracker_DelayTracking()
  if(addonInitialized - GetTime() < 0) then
    addonInitialized = 99999999999999;
    WorldBuffTrackerFrame:SetScript("OnUpdate", nil);
    for k, v in pairs(WorldBuffTrackerEvents) do
      WorldBuffTrackerFrame:RegisterEvent(k);
    end
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
    WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA();
    WorldBuffTracker_SetRemainingCD();
    if(WorldBuffTracker_HasFortuneCooldown()) then
      WorldBuffTracker_CreateDMFTimer();
    end
    WBT_CreateOptions();
    WBT_IncomingGUI_CreateGUI();
    WBT_GUI_CreateBarGUI();
    WBT_Incoming_TestActive();
  end
end

function WorldBuffTrackerEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, eventType, _,
      _, sourceName, _, _,
      destGUID, destName, _, _,
      spellId, spellName, _, auraType, _ = CombatLogGetCurrentEventInfo();
    if((eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH")
      and destName == playerName and auraType == "BUFF" and (WorldBuffTrackerSpells[spellName] ~= nil or strfind(spellName, L["DMFFortune"]))) then
        if(strfind(spellName, L["DMFFortune"])) then
          if(WorldBuffTracker_BuffIsValid(spellName, 2)) then
            WorldBuffTracker_CreateDMFTimer();
          end
          return
        end
        local buffID = WorldBuffTrackerSpells[spellName];
        if(buffID == 2) then
          if(WorldBuffTracker[realmName][faction]["2"]["Yell"] == WorldBuffTracker[realmName][faction]["3"]["Yell"]) then
            return
          else
            buffID = (WorldBuffTracker[realmName][faction]["2"]["Yell"] > WorldBuffTracker[realmName][faction]["3"]["Yell"]) and "2" or "3";
          end
        end
        if(WorldBuffTracker_BuffIsValid(spellName, buffID)) then
          if(IsInGuild()) then
            C_ChatInfo.SendAddonMessage("WBT-B", WorldBuffTracker_Serialize(buffID, GetServerTime(), VERSION), "GUILD");
          else
            WorldBuffTracker_HandleBuffById(buffID);
          end
        end
    elseif(eventType == "UNIT_DIED") then
      local npc = WorldBuffTrackerNPCs[faction][destName];
      if(npc) then
        WorldBuffTracker_BuffNPCDeath(npc);
      end
    end
end

function WorldBuffTrackerEvents:ZONE_CHANGED_NEW_AREA(...)
  WBT_GUI_ToggleTimer(WBT_GUI_ShouldDisplay());
  if(WorldBuffTrackerZones[faction][C_Map.GetBestMapForUnit("player")] ~= nil or
    WorldBuffTrackerZones["NEUTRAL"][C_Map.GetBestMapForUnit("player")] ~= nil) then
    WorldBuffTrackerFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:RegisterEvent("CHAT_MSG_MONSTER_YELL");
  else
    WorldBuffTrackerFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
    WorldBuffTrackerFrame:UnregisterEvent("CHAT_MSG_MONSTER_YELL");
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_MONSTER_YELL(msg, npc, ...)
  if(WorldBuffTrackerNPCs[faction][npc] == nil) then return end
  if(WorldBuffTrackerNPCs[faction][npc] == 5) then
      --print("NPC: ZG BUFFE - 5")
    if(strfind(msg, L["ZandalarYell"]) ~= nil) then
        --print("NPC: ZG BUFF CORRECT")
      SendChatMessage(wbtString .. "Spirit of Zandalar incoming!", "GUILD");
      WorldBuffTracker_BuffAnnounce("5", GetServerTime());
    else
          --print("INVALID MSG:|" .. msg)
          --print(L["ZandalarYell"])
    end
    return
  end
  npc = tostring(WorldBuffTrackerNPCs[faction][npc]);
  local yellID = WorldBuffTracker_GetYellID(npc, msg);
  local t = GetServerTime();
  if(yellID) then
    C_ChatInfo.SendAddonMessage("WBT-A", npc .. "_" .. VERSION, "GUILD");
    WorldBuffTracker_BroadcastIncomingData("YELL", npc, t, 3, yellID);
    WorldBuffTracker_BuffAnnounce(npc, t);
    WBT_Incoming_ProcessData(npc, t, yellID);
  end
end

function WorldBuffTracker_GetYellID(npc, msg)
    if(npc == "1") then
      if(strfind(msg, L["ThrallYell1"]) ~= nil) then
        return "Y1"
      elseif(strfind(msg, L["ThrallYell2"]) ~= nil) then
        return "Y2";
      end
    end
    if(npc == "2" or npc == "3") then
      local boss = npc == "2" and "Onyxia" or "Nefarian";
      if(strfind(msg, L[faction .. boss .. "1"]) ~= nil) then
        return "Y1"
      elseif(strfind(msg, L[faction .. boss .. "2"]) ~= nil) then
        return "Y2";
      end
    end
end

function WorldBuffTrackerEvents:QUEST_TURNED_IN(id, ...)
  if(WorldBuffTrackerQuests[faction][id]) then
    id = WorldBuffTrackerQuests[faction][id];
    local timestamp = GetServerTime() + WBT_Incoming_GetTimeToBuff(faction, id, "Y0");
    WorldBuffTracker_BroadcastIncomingData(false, id, timestamp, 3, "Y0");
  end
end

function WorldBuffTrackerEvents:CHAT_MSG_ADDON(prefix, message, channel, sender, ...)
  local sender, _ = strsplit("-", sender);
  local t = GetServerTime();
  if(prefix == "WBT-A") then
    if(channel == "GUILD") then
      local buff, version = strsplit("_", message);
      version = WorldBuffTracker_GetVersionFromString(version);
      if(WorldBuffTracker[realmName][faction][buff] ~= nil and version >= 11326) then
        WorldBuffTracker_BuffAnnounce(buff, t);
      end
    elseif(channel == "WHISPER" and GetServerTime() - WorldBuffTracker[realmName][faction]["LastSync"] < 60) then
      local version, timestamp = strsplit("_", message);
      WBT_Guild_InsertPlayer(sender, timestamp);
    end
  elseif(prefix == "WBT-B") then
    WorldBuffTracker_HandleBroadcast(message, sender);
  elseif(prefix == "WBT-R" and sender ~= playerName) then
    WorldBuffTracker_SendSyncData(sender);
  elseif(prefix == "WBT-I") then
    if(sender ~= playerName) then
      local version, buffID, timestamp, weight, yell = strsplit("_", message);
      if(channel == "GUILD") then
      else
        if(weight == "3") then
          weight = 2;
        end
      end
      if(WBT_Incoming_ProcessData(id, timestamp, yell)) then
        WorldBuffTracker_BroadcastIncomingData(channel, buffID, timestamp, weight, yell);
      end
    end
  elseif(prefix == "WBT-S") then
    if(channel == "GUILD" and sender ~= playerName) then
      if(message == "SYNC") then
        if(WBT_Guild_IsAnnouncer()) then
          WorldBuffTracker_SendSyncData(sender);
        end
        WBT_Guild_InsertPlayer(sender, GetServerTime());
        WBT_Guild_SendData(sender);
      else
        WorldBuffTracker_HandleSync(message, sender, true, channel);
      end
    elseif(channel == "YELL") then
      WorldBuffTracker_HandleSync(message, sender, false, channel);
    elseif(channel == "PARTY" or channel == "RAID") then
      WorldBuffTracker_HandleSync(message, sender, false, channel);
    elseif(channel == "WHISPER") then
      if(GetServerTime() - WorldBuffTracker[realmName][faction]["LastSync"] < 60) then
        WorldBuffTracker_HandleSync(message, sender, true, channel);
      else
        WorldBuffTracker_HandleSync(message, sender, false, channel);
      end
    end
  end
end

--function WorldBuffTrackerEvents:GOSSIP_SHOW(...)
  --local npc, _ = UnitName("npc");
  --if(WorldBuffTrackerNPCs[faction][npc]) then
    --coords,buffs

  --end
--end

function WorldBuffTrackerEvents:GROUP_JOINED(...)
  WorldBuffTracker_BroadcastChannel(WorldBuffTracker_BuildSyncMessage(), (IsInRaid() and "RAID" or "PARTY"));
  WorldBuffTracker_RequestGroup();
end

--function WorldBuffTrackerEvents:UNIT_TARGET(...)
  --if(not UnitPlayerControlled("target")) then
  --end
--end

function WorldBuffTracker_BuffAnnounce(buffID, t)
  buffID = tostring(buffID);
  if(buffID == "5") then
      local buffString = WorldBuffTrackerSpellsInverse[buffID];
      print(color["ORANGE"] .. wbtString .. buffString
      .. " incoming!|r");
      WorldBuffTracker_AlertPlayer(buffString);
    return
  end
  if(WorldBuffTracker[realmName][faction][buffID]["Yell"] < t) then
    WorldBuffTracker[realmName][faction][buffID]["Yell"] = t + 900;
    local buffString = WorldBuffTrackerSpellsInverse[buffID];
    print(color["ORANGE"] .. wbtString .. buffString
    .. " incoming!|r");
    WorldBuffTracker_AlertPlayer(buffString);
  end
end

local function WorldBuffTrackerFrame_Command(msg, editbox)
  local t = GetServerTime();
  print(color["PURPLE"] .. "------ WorldBuffTracker ------|r");
  if(WorldBuffTracker_HasFortuneCooldown()) then
    print(color["BLUE"] .. L["DMF"] .. "|r");
    print("    " .. color["ORANGE"] .. WorldBuffTracker_TimeUntilDate(math.floor(WBT_DMFData["Remaining"])) .. "|r");
  end
  for buff=1,3 do
    WorldBuffTracker_OutputByType(tostring(buff), t);
  end
  print(color["PURPLE"] .. "-----------------------------------|r")
end

function WorldBuffTracker_OutputByType(buff, t)
  local bT = WorldBuffTracker[realmName][faction][buff]["Timestamp"];
  local cd = WorldBuffTrackerBuffCDs[buff];
  print(color["BLUE"] .. WorldBuffTrackerSpellsInverse[buff]
   .. "|r");
    if(bT + cd > t) then

      --print("    " .. string.format(L["Popped: %s"], color["ORANGE"] .. date('%H:%M:%S', bT) .. "|r"));
      print("    " .. color["ORANGE"] .. date('%H:%M:%S', bT) .. "|r" .. color["PURPLE"] .. " → |r" .. color["GREEN"] .. date('%H:%M:%S', bT + cd) .. "|r");
      --print("    " .. string.format(L["Next: %s"], color["ORANGE"] .. date('%H:%M:%S', bT + cd) .. "|r"));
      print("    " .. color["ORANGE"] .. WorldBuffTracker_TimeUntilDate(bT - t + cd) .. "|r");
    else
      local tLast = t - bT - cd;
      if(tLast < 86400 and tLast > 0) then
        print("    " .. color["ORANGE"] .. "Window open for: ".. WorldBuffTracker_TimeUntilDate(tLast) .. "|r");
      else
        print("    ".. color["ORANGE"] .. L["No data"] .. "|r");
      end
    end
end

function WorldBuffTracker_TimeUntilDate(seconds)
  local str = "";
  if(seconds > 3600) then
    local hours = math.floor(seconds / 3600);
    str = hours .. "h ";
    seconds = seconds - 3600 * hours;
  end
  if(seconds > 60) then
    local minutes = math.floor(seconds / 60);
    str = str .. minutes .. "m ";
    seconds = seconds - 60 * minutes;
  end
  return str .. math.floor(seconds) .. "s";
end

function WorldBuffTracker_AlertPlayer(buff)
  if(BuffGUIVars["SilentInstances"] and IsInInstance()) then return end
  if(BuffGUIVars["PlaySound"]) then
    PlaySoundFile("Interface/AddOns/WorldBuffTracker/resources/sound/beware.ogg", "Master");
  end
  if(BuffGUIVars["SelfWhisper"]) then
    SendChatMessage(wbtString .. buff .. " incoming!", "WHISPER", nil, UnitName("player"));
  end
end

SLASH_WORLDBUFFS1, SLASH_WORLDBUFFS2, SLASH_WORLDBUFFS3 = '/worldbuffs', '/worldbufftracker', '/wbt'

SlashCmdList["WORLDBUFFS"] = WorldBuffTrackerFrame_Command;

WorldBuffTrackerFrame:SetScript("OnUpdate", WorldBuffTracker_Initialize);
WorldBuffTrackerFrame:SetScript("OnEvent", function(self, event, ...)
  WorldBuffTrackerEvents[event](self, ...);
end);
C_ChatInfo.RegisterAddonMessagePrefix("WBT-A");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-S");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-B");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-R");
C_ChatInfo.RegisterAddonMessagePrefix("WBT-I");
